//
// "$Id: CubeView.cxx,v 1.6 2004/10/17 20:05:03 Administrator Exp $"
//
// CubeView class implementation for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2003 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

#include "CubeView.h"
#include <math.h>
#include <stdio.h>

//the ansi standard does not include M_PI
#ifndef M_PI
#define M_PI 3.141592653589793238462643
#endif

using namespace std;

CubeView::CubeView(int x,int y,int w,int h,const char *l)
            : Fl_Gl_Window(x,y,w,h,l)
{
  char cwd[1024];
    size=1.0;
    xshift=0.0;
    yshift=0.0;

    windowHeight = h;
    windowWidth = w;
    sensitivity = MOUSE_YSENSITIVITY;

    //tyler's initializations
    hScale = 1.0;
    style = GL_LINE;
    zshift = 0.0;

    xpan = 0.0;
    ypan = 0.0;


    lineWidth = 1.0;
    ptSize = 1.0;

     //ini colors
    white[0] = 1.0;
    white[1] = 1.0;
    white[2] = 1.0;
    white[3] = 1.0;

    grey[0] = 0.5;
    grey[1] = 0.5;
    grey[2] = 0.5;
    grey[3] = 1.0;

    ambientColor[0] = 1.0;
    ambientColor[1] = 1.0;
    ambientColor[2] = 1.0;
    ambientColor[3] = 1.0;

    black[0] = 0.0;
    black[1] = 0.0;
    black[2] = 0.0;
    black[3] = 1.0;

    cyan[0] = 0.0;
    cyan[1] = 1.0;
    cyan[2] = 1.0;
    cyan[3] = 1.0;

    yellow[0] = 1.0;
    yellow[1] = 1.0;
    yellow[2] = 0.0;
    yellow[3] = 1.0;

    //defualt eye position
    moveEye(4.0, 6.0);

    spotHeight = 2.0;

    drawSources = false;
    drawGuides = false;

    sphereX = 0.0;

    //these should change according to the draw window size
    colWidth = 5.0;
    colHeight = 5.2;

    //load all directories up to current directory
    if (getcwd(cwd, 1024) != NULL) {
      string currentDir = string(cwd);
      string parent = getParentDir(currentDir);
      //printf("current dir: %s\n\n", cwd);

      cols.push_back(new DirColumn(getParentDir(parent), colWidth, colHeight, 
				   COLOFFSET - (colWidth + DIRCOL_SPACING) * 2, 0.0, 0.0));
      cols.push_back(new DirColumn(parent, colWidth, colHeight, 
				   COLOFFSET - (colWidth + DIRCOL_SPACING), 0.0, 0.0));
      
      cols.push_back(new DirColumn(currentDir, colWidth, colHeight, 
				   COLOFFSET, 0.0, 0.0));
    } 
}

//this draws each directory window for each column
void CubeView::drawMap(void) {
  //draw all columns
  //since the view is angled toward the first column, 
  //draw from first to last to keep transparencies lookin' sweet
  for (uint i = 0; i < cols.size(); i++) 
      cols[i]->draw();
  
  //change draw styles
  glPolygonMode(GL_FRONT_AND_BACK, style);
  glLineWidth(lineWidth);
  glPointSize(ptSize);

  if (drawGuides) {
    //draw the origin
    glBegin(GL_POINTS);
      glColor3f(1.0,0.0,0.0);
      glVertex3f(0.0,0.0,0.0);
    glEnd();
    
    //draw axis for debug
    //x
    glBegin(GL_LINES);
      glColor3f(1.0, 0.0, 0.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, yellow);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(20.0, 0.0, 0.0);
    glEnd();
    //y
    glBegin(GL_LINES);
      glColor3f(0.0, 1.0, 0.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, cyan);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 20.0, 0.0);
    glEnd();
    //z
    glBegin(GL_LINES);
      glColor3f(0.0, 0.0, 1.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, grey);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 20.0);
    glEnd();
  }


  //update the draw window
  glFlush();
  make_current();
  //swap_buffers();
}


void CubeView::draw() {
    if (!valid()) {
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glViewport(0,0,w(),h());
	//	glGetIntegerv(GL_VIEWPORT, vp);

        //glOrtho(-WINDOW_SIZE,WINDOW_SIZE,-WINDOW_SIZE,WINDOW_SIZE,-20030,10000)
	//this replaces the glOrtho with a perspective view
	gluPerspective(60.0, 1.0, 1, 10000);

	glMatrixMode(GL_MODELVIEW);

	//enable transparency
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_DEPTH_TEST);

	//make sure lighting effects are rendered relative to the viewer
	//glEnable(GL_LIGHTING);
	//glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	//glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientColor);

	antiAlias(true);
    }

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();
    glLoadIdentity();

    gluLookAt(eyePos[0], eyePos[1] + 2.0, eyePos[2],
	      0.0, 0.0, 0.0, UP_X, UP_Y, UP_Z);

    // glMultMatrixf(the_rotMatrix);
    glScalef(float(size),
    	     float(size),
    	     float(size));


    //glClearStencil(0x0);
    //glEnable(GL_STENCIL_TEST);


    //render lights if there is something to draw
    //    if (maxHeight != MIN_NOT_SET)
    //renderLights(maxHeight * hScale + zshift);
    
    //turn on ambient light
    //ambient(1);

    //translate
    //glTranslatef(xshift + xpan, yshift + ypan, zshift * hScale);  

    //glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
    //glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    //glEnable(GL_LIGHTING);

    //finally draw the objects
    drawMap();

    glPopMatrix();
}


//draws three lights on the screen and ambient light
//the parameter is the maximum height of the map
//GL_LIGHT0: yellow spotlight shining on floor (distict yellow spot)
//GL_LIGHT1: cyan positional light above and on one side of the ground grid
//GL_LIGHT2: white directional light shining straight down
//GL_LIGHT3: ambient light????
void CubeView::renderLights(GLdouble height) {
  GLfloat light0pos[] = {0.0, 0.0, height + spotHeight * hScale, 1.0};
  GLfloat light1pos[] = {0.0, 0.0, height + 10.0, 1.0}; //{rows, cols / 2.0, height + 10.0, 1.0};
  GLfloat light2pos[] = {0.0 , 0.0, height + 1.0, 0.0};

  GLfloat light0dir[] = {0.0, 0.0, -1.0};

  //printf("rendering lights with height %f\n", height);

  //draw ambient white light
  //glLightModelfv(GL_LIGHT_MODEL_AMBIENT, white);

  //draw the light positions
  if(drawSources) {
    glBegin(GL_POINTS);
      //yellow light
      glColor3f(1.0,1.0,0.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, yellow);
      glVertex3fv(light0pos);
      
      //cyan positional
      glColor3f(0.0,1.0,1.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, cyan);
      glVertex3fv(light1pos);
      
      //white directional
      glColor3f(1.0,1.0,1.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, white);
      glVertex3fv(light2pos);
    glEnd();

    /*draw spotlight vector
    glBegin(GL_LINES);
      glColor3f(1.0,1.0,0.0);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, yellow);
      glVertex3fv(light0pos);
      glVertex3fv(light0dir);
      glEnd();*/
  }

  //yellow spotlight shining on floor (distict yellow spot)
  glLightfv(GL_LIGHT0, GL_POSITION, light0pos);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, yellow);
  glLightfv(GL_LIGHT0, GL_SPECULAR, yellow);
  glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light0dir);
  glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 15.0);
  glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 3.0);

  //cyan positional light above and on one side of the ground grid
  glLightfv(GL_LIGHT1, GL_POSITION, light1pos);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, cyan);
  glLightfv(GL_LIGHT1, GL_SPECULAR, cyan);


  //draw white directional light shining straight down
  glLightfv(GL_LIGHT2, GL_POSITION, light2pos);
  glLightfv(GL_LIGHT2, GL_DIFFUSE, white);
  glLightfv(GL_LIGHT2, GL_SPECULAR, white);



}


//sets up selection mode and fills the pickBuffer 
//with hits defined inside the rectangle centered at x, y.
//returns the number of hits.
int CubeView::setupSelection(GLdouble x, GLdouble y, GLdouble width, GLdouble height,
			     GLuint pickBuffer[]) {
  //GLuint pickBuffer[PICK_BUF_SIZE];
  GLint vpArray[4];

  memset(pickBuffer,0,PICK_BUF_SIZE*sizeof(GLuint));

  //start picking mode
  glSelectBuffer(PICK_BUF_SIZE, pickBuffer);
  glRenderMode(GL_SELECT);
  glInitNames();

  //save current viewing matrix
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();

  //set up 1x1 pick window.
  //must invert mouse value, but this is already done
  glGetIntegerv(GL_VIEWPORT, vpArray);
  gluPickMatrix((GLdouble)x, (GLdouble)y, width, height, vpArray);

  //set up window and draw all the objects in the pick matrix
  gluPerspective(60.0, 1.0, 1, 10000);
  glMatrixMode(GL_MODELVIEW);
  
  glPushMatrix();
  glLoadIdentity();
  
  gluLookAt(eyePos[0], eyePos[1] + 2.0, eyePos[2],
	    0.0, 0.0, 0.0, UP_X, UP_Y, UP_Z);
  
  glScalef(float(size), float(size), float(size));
  
  //check selection for each column
  for (uint i = 0; i < cols.size(); i++) {
    glPushName(i);
    (cols[i])->draw();
    glPopName();
  }

  //restore viewing matrix
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();

  glFlush();

  //get number of picks and change to normal render mode
  return glRenderMode(GL_RENDER);
}

//this handles the selection of a object
void CubeView::onClick(int button, int x, int y) {
  GLuint pickBuffer[PICK_BUF_SIZE];
  GLint numPicks;
  //these hold the results
  vector<void*> pickStacks;
  int name;

  numPicks = setupSelection((GLdouble)x, (GLdouble)y, 1.0, 1.0, pickBuffer);

  name = processHits(numPicks, pickBuffer, &pickStacks);

  //quit if no hits
  if (name == -1)
    return;

  string dir = cols[name]->onClick(button, pickStacks, 1);
  //make new directory col after the current one
  if (dir != "") {
    if (isDir(dir)) {
      float x;
      bool erasedCols;

      //printf("open: %s\n", dir.c_str());
      
      //erase all columns after the one clicked
      for (uint j = cols.size(); int(j) > name + 1; j--) {
	cols.pop_back();
	erasedCols = true;
      }
      
      //make x pos of next column so the new one will take its place
      x = cols[name]->x() + colWidth + DIRCOL_SPACING;
      //printf("x: %f\n", x);
      cols.push_back(new DirColumn(dir, colWidth, colHeight, x, 0.0, 0.0));
      
      //if the window clicked was the last or at the end of the screen, move over baby!
      if (!erasedCols || isPastRight(cols.back()))
	strafeCols(-(colWidth + DIRCOL_SPACING));

      redraw();
    }
    else {
      printf("open file: %s\n", dir.c_str());
    }
  }

}

//returns the index of the column at mouse position x.
//if no column is there, returns -1.
int CubeView::getCurrentCol(int x, int y) {
  GLuint pickBuffer[PICK_BUF_SIZE];
  GLint numPicks;
  //this hold the results
  vector<void*> pickStacks;

  numPicks = setupSelection((GLdouble)x, (GLdouble)y, 1.0, windowHeight, pickBuffer);

  return processHits(numPicks, pickBuffer, &pickStacks);
}

//this function taken from the Redbook, pg 369
//it returns the index of the column clicked, or -1 if none were selected.
//it adds the stacktrace for each hit into pickStacks.
/* processHits prints out the contents of the selection array.*/
int CubeView::processHits (GLint hits, GLuint buffer[], vector<void*> *pickStacks) {
  int i;
  uint j, frontmost;
  GLuint names, *ptr;
  vector<uint> *currentStack;
  vector<float> stacksZ;
  
  //printf ("hits = %d\n", hits);
  ptr = (GLuint *) buffer;

  //move on if there were no hits
  if (hits == 0)
    return -1;

  //build each pickStack from results in buffer and add to pickStacks
  for (i = 0; i < hits; i++) { /* for each hit */
    names = *ptr;
    //printf (" number of names for this hit = %d\n", names);
    ptr++;

    //add z value for this stack
    stacksZ.push_back((float) *ptr/0x7fffffff);

    //printf(" z1 is %g;", (float) *ptr/0x7fffffff);
    ptr++;
    //printf(" z2 is %g\n", (float) *ptr/0x7fffffff);
    ptr++;
    //printf (" names are ");

    //add all names for this hit to a stack
    currentStack = new vector<uint>;
    for (j = 0; j < names; j++) { /* for each name */
      //printf ("%d ", *ptr);

      currentStack->push_back(*ptr);

      ptr++;
    }

    pickStacks->push_back((void*)currentStack);

    //printf ("\n");
  }

  //find column that was clicked on and pass processing to it.
  //if it returns a non-empty string, move to that directory.
  //first must find the frontmost column (lowest Z value)
  frontmost = 0;
  for (j = 0; j < stacksZ.size(); j++) 
    if (stacksZ[frontmost] > stacksZ[j])
      frontmost = j;

  currentStack = (vector<uint>*)(*pickStacks)[frontmost];

  return (*currentStack)[0];
}

//returns true if d is past the right edge of the window
inline bool CubeView::isPastRight(DirColumn* d) {
  return (d->x() - d->getWidth() / 2.0 > windowWidth / WINDOW_SCALE / size);
}

//this animates moving the columns left or right.
void CubeView::strafeCols(float amount) {
  float deltax = 0.0;  
  //float time = 0.0;

  deltax = amount / ANIMATION_SPEED;
  for (int i = 0; i < ANIMATION_SPEED; i++) {
    for (vector<DirColumn*>::iterator iter = cols.begin(); iter != cols.end(); iter++) 
      (*iter)->x((*iter)->x() + deltax);

    flush();
  }
}

int CubeView::handle(int event)
{
    bool handled = false;
    int x = Fl::event_x();
    int y = windowHeight - Fl::event_y();

    //printf("handled called %d %d last (%d %d)\n", x, y, lastMouseX, lastMouseY);

    //handle keypresses
    if (Fl::event_state(FL_CTRL)) {
      float shuffleSensitivity = windowHeight / MAX_WINDOWS_VISIBLE * sensitivity;  
      //float colSensitivity = windowWidth / (colWidth * 3) * sensitivity;
      float deltay = y - lastMouseY;
      float deltax = x - lastMouseX;

      if (abs(deltay) >= abs(deltax)) {
	//find current dir column
	int currentCol = getCurrentCol(x, y);

	//move current dir column forward or backward
	if (currentCol >= 0)
	  cols[currentCol]->shuffle(float(deltay / shuffleSensitivity));
      }
      else {
	//move all dir columns horizontally

	//pad user's movement to a fraction of the window
	if (abs(x - xPadBegin) > windowWidth * MOUSE_XSENSITIVITY) {
	  xPadBegin = x;

	//move right
	if (deltax > 0)
	  strafeCols(colWidth + DIRCOL_SPACING);
	//move left
	if (deltax < 0)
	  strafeCols(-colWidth - DIRCOL_SPACING);	  
	}
      }

      lastMouseX = x;
      lastMouseY = y;

      /* dont need this anymore      
      //check to see if the current directory column is leaving the right side.
      //if so, remove it and make its parent the current.
      //printf("curx: %f windowWidth: %d\n", (cols.back())->x(), windowWidth);
      if (isPastRight(cols.back())) {
	if (cols.size() > 1)
	  cols.pop_back();
      }
      */
      
      redraw();      
      return true;
    }

    lastMouseX = x;
    lastMouseY = y;
    xPadBegin = x;

    /* Handle each button */
    //printf("button: %d, event: %d\n", Fl::event_button(), event);
    if (Fl::event_button() > 0) {
      //this requires a little hack since the middle scroll wheel does
      //not generate events.    
      if (Fl::event_button() == MOUSE_SCROLL_UP ||
	  Fl::event_button() == MOUSE_SCROLL_DOWN) {
	//process click and take appropriate action
	if (event == FL_PUSH) {
	  onClick(Fl::event_button(), x, y);
	  redraw(); 
	  return true;
	}
	else 
	  return false;
      }
      else {
	switch (event) {
	case FL_PUSH:
	  //process click and take appropriate action
	  onClick(Fl::event_button(), x, y);
	  
	  redraw();
	  return true;
	  break;
	  
	case FL_RELEASE:
	  return true;
	  break;
	  
	case FL_DRAG:
	  
	  return true;
	  break;
	}
      }
    }
    else
      handled = false;    
    	

    /* If we handled it, return 1 */
    if (handled)
    {
	redraw();
	return 1;
    }
    /* If we didn't handle it, pass it on */
    return Fl_Gl_Window::handle(event);
}

//////////////////////////////////////////////////////////////////////
//the rest of the GUI calls
//////////////////////////////////////////////////////////////////////

//this loads an input file from filename
void CubeView::load(char* filename) {

}

//these change the drawing style (solid, wireframe, points)
void CubeView::drawStyle(GLenum s) {
  style = s;
  redraw();
}

void CubeView::lineThickness(float size) {
  lineWidth = size;
  redraw();
}

void CubeView::pointSize(float size) {
  ptSize = size;
  redraw();
}

void CubeView::heightScale(float scale) {
  hScale = scale;
  redraw();
}

void CubeView::antiAlias(bool on) {
  //change anti-aliasing
  if (on)
    glEnable(GL_LINE_SMOOTH);
  else
    glDisable(GL_LINE_SMOOTH);

  redraw();
}

//this function turns the specified light on or off
void CubeView::lightSwitch(GLenum light, bool on) {
  if (on)
    glEnable(light);
  else
    glDisable(light);

  redraw();
}

//turns on and off the  ambient white light
void CubeView::ambient(bool on) {
  if (on)
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientColor);
  else
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, black);

  //printf("ambient changed\n");

  redraw();
}


void CubeView::moveEye(float theta, float omega) {
  //if moving in the x direction, change x to user specified value
  //and calculate z so that we travel in a circle, always 20 away from
  //the origin.
  eyePos[0] = (GLfloat)(EYE_DISTANCE * cos(theta) * sin(omega));
  eyePos[1] = (GLfloat)(EYE_DISTANCE * sin(theta) * sin(omega));
  eyePos[2] = (GLfloat)(EYE_DISTANCE * cos(omega));

  //printf("moveEye(%f, %f, %f) (%f %f)\n", eyePos[0], eyePos[1], eyePos[2], omega, theta);
  redraw();
}

//
// End of "$Id: CubeView.cxx,v 1.6 2004/10/17 20:05:03 Administrator Exp $".
//
