//This is a generic object to draw
#include "DirColumn.h"
#include <GL/glu.h>

////////////////
//Constructor //
////////////////
DirColumn::DirColumn() {
  centerx = 0.0;
  centery = 0.0;
  centerz = 0.0;
  
  width = 0.0;
  height = 0.0;
};

DirColumn::DirColumn(string dir, float w, float h, float x, float y, float z) {
  FileWindow* temp;
  float zchange, ychange;
  vector<string> dirs;
    
  //initialize geometry
  centerx = x;
  centery = y;
  centerz = z;
  
  width = w;
  height = h;
  
  directory = normalizeDir(dir);
  
  //initalize the directory list
  dirs = getDirList(directory, GETDIRS);
  
  //add the current directory to the list, as an index
  dirs.insert(dirs.begin(), directory);

  //make a filewindow for each subdirectory and line them up
  zchange = z;
  ychange = y;
  //printf("dir %s\n", dir.c_str());
  for (vector<string>::iterator i = dirs.begin(); i != dirs.end(); i++) {
    temp = new FileWindow(*i, width, height, x, ychange, zchange);

    //the index window is at the beginning of the stack
    if (i == dirs.begin())
      temp->isIndex = true;
    else
      temp->isIndex = false;

    col.push_back(temp);
    zchange -= WINDOW_ZSPACING;
  }
  
  //set the first directory to be in front
  if (col.size() > 0) {
    current = *(col.begin());
    current->isCurrent = true;
  }
};

//////////////
//Mutators: //
//////////////
//move the object's center point in space
void DirColumn::x(float val) {
  centerx = val;

  //update all child windows
  for (vector<FileWindow*>::iterator i = col.begin(); i != col.end(); i++) 
    (*i)->x(val);
}

void DirColumn::y(float val) {
  centery = val;

  //update all child windows
  for (vector<FileWindow*>::iterator i = col.begin(); i != col.end(); i++) 
    (*i)->y(val);
}

void DirColumn::z(float val) {
  centerz = val;

  //update all child windows
  for (vector<FileWindow*>::iterator i = col.begin(); i != col.end(); i++) {
    (*i)->z(val);
    val += WINDOW_ZSPACING;
  }
}

//change the size of the object
void DirColumn::setWidth(float val) {
  width = val;
}

void DirColumn::setHeight(float val) {
  height = val;
}


//this shuffles through the separate directory windows.
//they wrap to the back once they hit the front (designated by this class' z member). 
//the parameter is how much to move in the z direction
void DirColumn::shuffle(float dist) {
  vector<FileWindow*>::iterator i;
  //assumes windows are very thin
  const float minz = centerz - col.size() * WINDOW_ZSPACING;

  //printf("\tshuffle(%f) minz = %f\n", dist, minz);
 
  //reset current pointer
  if (col.size() > 0)
    current = *(col.begin());

  /*  //move each window, wrapping if it goes past the edge
  for (i = col.begin(); i != col.end(); i++) {
    (*i)->isCurrent = false;
  }
  */
  //move each window, wrapping if it goes past the edge
  for (i = col.begin(); i != col.end(); i++) {
    float currentz = (*i)->z();

    if (currentz + dist > centerz) 
      (*i)->z(float(minz + (dist - (centerz - currentz)))); 
    else if (currentz + dist < minz) 
      (*i)->z(centerz - (minz - (currentz + dist))); 
    else //dont wrap
      (*i)->z(currentz + dist);

    //update the current pointer with frontmost object
    (*i)->isCurrent = false;
    if (/*currentz + dist*/(*i)->z() >= current->z()) {
      current->isCurrent = false;
      current = *i;
      current->isCurrent = true;
      // printf("new current %s\n", (current->getDir()).c_str());
    }
  }



}

//////////////
//Getters:  //
//////////////
//get the object's center point in space
float DirColumn::x(void) {
  return centerx;
}

float DirColumn::y(void) {
  return centery;
}

float DirColumn::z(void) {
  return centerz;
}

//get the size of the object
float DirColumn::getWidth(void) {
  return width;
}

float DirColumn::getHeight(void) {
  return height;
}

//////////////
// Events:  //
//////////////
//given the pickStacks, finds what was clicked on and proceeds as necessary.
//it only processes each pickStack from depth on.
//if an icon was clicked, returns that icon's filename.
//otherwise returns "".
string DirColumn::onClick(int button, vector<void*> pickStacks, uint depth) {
  vector<uint> *currentStack;
  string result;

  if (button == MOUSE_SCROLL_UP || button == MOUSE_SCROLL_DOWN ) 
    return current->onClick(button, pickStacks, depth + 1);
  else {    
    for(uint j = 0; j < pickStacks.size(); j++) {
      currentStack = (vector<uint>*)pickStacks[j];
      
      //search for icons that were clicked on
      if (currentStack->size() > depth + 1) {
	uint name = (*currentStack)[depth];
	//printf("col[%d]\n", name);
	result = col[name]->onClick(button, pickStacks, depth + 1);
	
	//when clicking a dir in the index, we want to move to that directory
	//in the column, not open a redundant column
	if (col[name]->isIndex && isDir(result)) {
	  uint i;
	  //find the directory to move to
	  for (i = 0; i < col.size(); i++)
	    if (col[i]->getDir() == result) break;
	  
	  shuffle((centerz - col[i]->z() - 0.1));
	  return "";
	}
	else
	  return result;
      }
    }
  }

  return "";	
}


//this draws each filewindow from last to the current window
void DirColumn::draw(void) {
  bool startDraw = false;
  uint i;

  for (i = col.size() - 1; i >= 0 ; i--) {
    if (startDraw) {
      glPushName(i);
      (col[i])->draw();
      glPopName();

      if (col[i] == current) 
	break;
    }
    else //once we see the current one, start drawing
      if (col[i] == current)
	startDraw = true;

    //wrap around
    if (i == 0)
      i = col.size();
  }
}
