//This is a generic object to draw
#include "FileWindow.h"
#include "writeText.h"
#include <stdio.h>

//font for the window
#define FONT GLUT_BITMAP_HELVETICA_10

//this are the colors of the window elements
float FileWindow::background[4] = {1.0, 1.0, 1.0, 0.4};
float FileWindow::titlebar[4] = {0.4, 0.6, 1.0, 0.4};
float FileWindow::indextitlebar[4] = {1.0, 0.2, 0.0, 0.4};
float FileWindow::titlebartext[4] = {0.0, 1.0, 1.0, 1.0};
float FileWindow::activebackground[4] = {1.0, 1.0, 1.0, 1.0};
float FileWindow::activetitlebar[4] = {0.0, 0.2, 1.0, 1.0};
float FileWindow::activeindextitlebar[4] = {1.0, 0.2, 0.0, 1.0};

//load texture for the window
GLuint FileWindow::winTexture = 0;
GLuint FileWindow::barTexture = 0;

////////////////
//Constructor //
////////////////
FileWindow::FileWindow() {
  centerx = 0.0;
  centery = 0.0;
  centerz = 0.0;
  
  width = 0.0;
  height = 0.0;
  
  isCurrent = false;
};

FileWindow::FileWindow(string dir, float w, float h, float x, float y, float z) {
  centerx = x;
  centery = y;
  centerz = z;
  
  width = w;
  height = h;
  
  isCurrent = false;
  
  directory = dir;
  
  loadIcons();

};

//////////////
//Mutators: //
//////////////
//move the object's center point in space
void FileWindow::x(float val) {
  centerx = val;
}

void FileWindow::y(float val) {
  centery = val;
}

void FileWindow::z(float val) {
  centerz = val;
}

//change the size of the object
void FileWindow::setWidth(float val) {
  width = val;
}

void FileWindow::setHeight(float val) {
  height = val;
}

void FileWindow::setDir(string dir) {
  directory = dir;
  loadIcons();
}

//////////////
//Getters:  //
//////////////
//get the object's center point in space
float FileWindow::x(void) {
  return centerx;
}

float FileWindow::y(void) {
  return centery;
}

float FileWindow::z(void) {
  return centerz;
}

//get the size of the object
float FileWindow::getWidth(void) {
  return width;
}

float FileWindow::getHeight(void) {
  return height;
}

string FileWindow::getDir(void) {
  return directory;
}

//////////////
// Events:  //
//////////////

//given the pickStacks, finds what was clicked on and proceeds as necessary.
//it only processes the pickStacks from depth on.
//if an icon was clicked, returns that icon's filename.
//otherwise returns "".
//if the button is the scroll wheel, scroll the icons accordingly.
string FileWindow::onClick(int button, vector<void*> pickStacks, uint depth) {

  //printf("button: %d\n", button);
  if(button == MOUSE_SCROLL_DOWN) {
    scrollIcons(int(ICON_COLS));
  }
  else if(button == MOUSE_SCROLL_UP) {
    scrollIcons(int(-ICON_COLS));
  }
  else {
    vector<uint> *currentStack;
    
    for(uint j = 0; j < pickStacks.size(); j++) {
      currentStack = (vector<uint>*)pickStacks[j];
      
      //search for icons that were clicked on
      if (currentStack->size() > depth) {
	uint name = (*currentStack)[depth];
	//printf("icons[%d] (out of %d)\n", name, icons.size());
	if (icons.size() >= name)
	  return icons[name]->getFullPath();
      }
    }
  }

  return "";
}

//////////////
// The Rest://
//////////////

//renders the window
void FileWindow::draw(void) {
  float topOfWindow = height / 2.0 - TITLEBAR_HEIGHT * height;

  //load textures the first time only
  if (winTexture == 0) {
    winTexture = LoadTextureRAW("textures/window.raw", 256, 256, 1);
    barTexture = LoadTextureRAW("textures/titlebar.raw", 256, 30, 1);
  }

  glPushMatrix();

  glTranslatef(centerx, centery, centerz);

  //change draw styles
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glLineWidth(1.0);
  glPointSize(1.0);
	
  //draw window background
  if (isCurrent)
    glColor4fv(activebackground);
  else
    glColor4fv(background);

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, winTexture);

  glBegin(GL_POLYGON);
    glTexCoord2f(0.0, 0.0);    glVertex2f(-width / 2.0, -height / 2.0);
    glTexCoord2f(1.0, 0.0);    glVertex2f(width / 2.0, -height / 2.0);
    glTexCoord2f(1.0, 1.0);    glVertex2f(width / 2.0, topOfWindow);
    glTexCoord2f(0.0, 1.0);    glVertex2f(-width / 2.0, topOfWindow);
  glEnd(); 

  glDisable(GL_TEXTURE_2D);

  //draw titlebar
  if (isCurrent) {
    if (isIndex)
      glColor4fv(activeindextitlebar);
    else
      glColor4fv(activetitlebar);
  }
  else {
    if (isIndex)
      glColor4fv(indextitlebar);
    else
      glColor4fv(titlebar);
  }

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, barTexture);

  glBegin(GL_POLYGON);
    glTexCoord2f(0.0, 0.0);    glVertex2f(-width / 2.0, topOfWindow);
    glTexCoord2f(1.0, 0.0);    glVertex2f(width / 2.0, topOfWindow);
    glTexCoord2f(1.0, 1.0);    glVertex2f(width / 2.0, height / 2.0);
    glTexCoord2f(0.0, 1.0);    glVertex2f(-width / 2.0, height / 2.0);
  glEnd(); 

  glDisable(GL_TEXTURE_2D);
	
  //draw titlebar text
  writeText(getBaseName(directory), NULL, titlebartext, 
	    -width / 2.0 + TITLEBAR_HPADDING, 
	    height / 2.0 /*- TITLEBAR_VPADDING*/, 0.01, width, 
	    TITLEBAR_HEIGHT * height - TITLEBAR_VPADDING);

  // printf("%s: (%f, %f, %f) height: %f\n",
  //	 directory.c_str(), centerx, centery, centerz, TITLEBAR_HEIGHT * height);

  //draw icons   
  glPushMatrix();
  if (isCurrent) {
    //draw from the current icon to the last that will fit in the window
    
    float winHeight = height - (height * TITLEBAR_HEIGHT) - ICON_SPACING - iconwidth / 2.0;
    uint iconRows = uint(winHeight / (iconwidth + ICON_SPACING));
    uint lastIcon = uint(currentIcon + iconRows * ICON_COLS - 1);
    for (uint i = currentIcon; i < icons.size() && i <= lastIcon; i++) {
      glPushName(i);
      (icons[i])->draw();
      glPopName(); 
    }   
  }
  glPopMatrix();

  glPopMatrix();
};

void FileWindow::loadIcons(void) {
  float h, w;
  vector<string> dirList;
  vector<string> fileList;
  vector<string>::iterator i;
  
  //construct the list of all files and subdirectories and make icons
  dirList = getDirList(directory, GETDIRS);
  fileList = getDirList(directory, GETFILES);
  //printf("filedir: %s\n", directory.c_str());
  //append all files onto the end of the directory list so we only have
  //one list to render
  for (i = fileList.begin(); i != fileList.end(); i++) 
    dirList.push_back(*i);
  
  iconwidth = (width - ICON_SPACING * (ICON_COLS + 1.0)) / ICON_COLS;
  
  h = height / 2.0 - (TITLEBAR_HEIGHT * height) - ICON_SPACING - iconwidth / 2.0;
  w = -width / 2.0;
  
  //printf("iconwidth: %f width: %f\n", iconwidth, width);
  for (i = dirList.begin(); i != dirList.end(); i++) {
    w += ICON_SPACING + iconwidth;
    //wrap icons if past the right edge
    if (w + iconwidth / 2.0 > width / 2.0 - ICON_SPACING) {
      w = -width / 2.0 + iconwidth + ICON_SPACING;
      h -= ICON_SPACING + iconwidth; //icon width is same as its height
    }
    //printf("\t\tadded icon: %s\n", (*i).c_str());
    icons.push_back(new Icon(*i, iconwidth, iconwidth, w, h, 0.0));
  }

  currentIcon = 0;
}

void FileWindow::scrollIcons(int amount) {
  int current;
  float hChange;

  //change currentIcon pointer
  if (amount < 0) {
    hChange = float((iconwidth + ICON_SPACING)); 

    current = int(currentIcon + ICON_COLS - 1);
    if ((uint)current >= icons.size()) {
      //currentIcon = icons.size()- 1;
      hChange = 0.0;
    }
    else
      currentIcon = current;
  }
  else {
    hChange = float(-(iconwidth + ICON_SPACING));

    current = int(currentIcon - ICON_COLS +1);
    if (current < 0) {
      //currentIcon = 0;
      hChange = 0.0;
    }
    else
      currentIcon = current;
  }

  //hChange = float((iconwidth + ICON_SPACING) * float(oldCurIcon - currentIcon) / ICON_COLS);
  //printf("hC: %f oldCur: %d, cur: %d\n", hChange, oldCurIcon, currentIcon);
  //move all icons up
  for (uint i = 0; i < icons.size(); i++) 
    icons[i]->y( icons[i]->y() + hChange );

}
