//This is a icon of a file or a directory.
#include "Icon.h"
#include "writeText.h"
#include "dir.h"

//these are the colors of the icon
//float Icon::dircolor[4] = {0.8, 0.6, 0.0, 1.0};
float Icon::textcolor[4] = {0.0, 0.6, 0.2, 1.0};
float Icon::filecolor[4] = {1.0, 1.0, 1.0, 1.0};
float Icon::filebordercolor[4] = {0.0, 0.0, 0.0, 1.0};
float Icon::filetextcolor[4] = {0.0, 0.0, 1.0, 1.0};

GLuint Icon::iconTexture = 0;

////////////////
//Constructor //
////////////////
Icon::Icon() {
  centerx = 0.0;
  centery = 0.0;
  centerz = 0.0;
  
  width = 0.0;
  height = 0.0;
  
  isADir = false;
};

Icon::Icon(string dir, float w, float h, float x, float y, float z) {
  centerx = x;
  centery = y;
  centerz = z + 0.1;
  
  width = w;
  height = h;
  
  isADir = false;
  
  path = dir;
  
  //now must parse filename from full path
  name = getBaseName(path);
  //printf("icon(%s)\n", dir.c_str());

  dircolor[0] = 0.0;
  dircolor[1] = 0.0;
  dircolor[2] = 0.3;
  dircolor[3] = 1.0;

  //find file and determine if it is a directory
  if (isDir(path)) {
    float numFiles, numDirs, total;
    vector<string> fileList;
    
    isADir = true;
    
    //directories are proportional: The ratio of files to directories
    //contained within them are represented by the ratio of the lengths
    //of the top and bottom, respectively.
    //Also, the color varies according to how many files and directories
    //it contains. It is a dull blue when there are none, and a bright red
    //when there are more than 20.
    
    //find ratios 
    fileList = getDirList(path, GETFILES);
    numFiles = fileList.size();
    fileList = getDirList(path, GETDIRS);
    numDirs = fileList.size();
    
    total = numDirs + numFiles;
    if (total <= 0.0) {
      top = width / SMALLESTDIRDIVISION;
      bottom = top;
    }
    else {
      top = width * numFiles / total;
      bottom = width * numDirs / total;
    }

    dircolor[0] = numDirs / MAXFILES;
    dircolor[1] = 0.0;
    dircolor[2] = 0.3;
    dircolor[3] = 1.0;

    dirtopcolor[0] = 0.0;
    dirtopcolor[1] = numFiles / MAXFILES;
    dirtopcolor[2] = 0.3;
    dirtopcolor[3] = 1.0;

  }
};


//////////////
//Mutators: //
//////////////
//move the object's center point in space
void Icon::x(float val) {
  centerx = val;
}

void Icon::y(float val) {
  centery = val;
}

void Icon::z(float val) {
  centerz = val;
}

//change the size of the object
void Icon::setWidth(float val) {
  width = val;
}

void Icon::setHeight(float val) {
  height = val;
}

//////////////
//Getters:  //
//////////////
//get the object's center point in space
float Icon::x(void) {
  return centerx;
}

float Icon::y(void) {
  return centery;
}

float Icon::z(void) {
  return centerz;
}

//get the size of the object
float Icon::getWidth(void) {
  return width;
}

float Icon::getHeight(void) {
  return height;
}

string Icon::getFullPath(void) {
  return path;
}

//////////////
// The Rest://
//////////////
void Icon::draw(void) {
  const float bottomOfGlyph = height / 2.0 - ICONHEIGHT * height;
  float glyphWidth;

  glPushMatrix();
 
  glTranslatef(centerx, centery, centerz);

  //change draw styles
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glLineWidth(1.0);
  glPointSize(1.0);

  /*  //draw a test height line
  glColor3f(1.0,0.0,0.0);
  glBegin(GL_LINES);
    glVertex3f(-width / 2.0, height / 2.0, 0.0);
    glVertex3f(width / 2.0, -height / 2.0, 0.0);
  glEnd();
  */

  //draw icon
  if (isADir) {
    if (iconTexture == 0)
      iconTexture = LoadTextureRAW("textures/diricon.raw", 256, 256, 1);

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, iconTexture);

    //draw a quad with the appropriate ratio lengths
    glBegin(GL_POLYGON);
      glColor4fv(dirtopcolor);
      glTexCoord2f(0.0, 1.0);   glVertex2f(-top / 2.0, height / 2.0);
      glTexCoord2f(1.0, 1.0);   glVertex2f(top / 2.0, height / 2.0);

      glColor4fv(dircolor);
      glTexCoord2f(1.0, 0.0);   glVertex2f(bottom / 2.0, bottomOfGlyph);
      glTexCoord2f(0.0, 0.0);   glVertex2f(-bottom / 2.0, bottomOfGlyph);
    glEnd();

    glDisable(GL_TEXTURE_2D);

  }
  else {
    float spacing = (height * ICONHEIGHT) / (NUMOFTEXTLINES + 1.0);

    glyphWidth = height * ICONHEIGHT * ICONWIDTH;

    //draw file background with a bent corner
    glColor4fv(filecolor);
    glBegin(GL_POLYGON);	
      glVertex2f(-glyphWidth / 2.0 + spacing, height / 2.0);
      glVertex2f(glyphWidth / 2.0, height / 2.0);
      glVertex2f(glyphWidth / 2.0, bottomOfGlyph);
      glVertex2f(-glyphWidth / 2.0, bottomOfGlyph);
      glVertex2f(-glyphWidth / 2.0, height / 2.0 - spacing);
    glEnd(); 

    //draw the other stuff on top
    glTranslatef(0.0, 0.0, 0.01);

    //draw border with a bent corner
    glColor4fv(filebordercolor);
    glBegin(GL_LINE_LOOP);	
      glVertex2f(-glyphWidth / 2.0 + spacing, height / 2.0);
      glVertex2f(glyphWidth / 2.0, height / 2.0);
      glVertex2f(glyphWidth / 2.0, bottomOfGlyph);
      glVertex2f(-glyphWidth / 2.0, bottomOfGlyph);
      glVertex2f(-glyphWidth / 2.0, height / 2.0 - spacing);
    glEnd(); 

    //draw text lines
    glLineWidth(2.0);
    glColor4fv(filetextcolor);
    for (int i = 1; i <= NUMOFTEXTLINES; i++) {
      glBegin(GL_LINES);	
        glVertex2f(-glyphWidth / 2.0 + FILELINESMARGIN, height / 2.0 - spacing * i);
	glVertex2f(glyphWidth / 2.0 - FILELINESMARGIN, height / 2.0 - spacing * i);
      glEnd();
    }
  }
  
  //draw text under the icon
  writeText(name, NULL, textcolor, -width / 2.0, bottomOfGlyph + TEXTMARGIN,
	    0.01, width, (1.0 - ICONHEIGHT) * height - 2 * TEXTMARGIN);
  
  glPopMatrix();
}
