//These functions return a vector containing each directory entry.
//It either returns the subdirectories, or the files in that directory,
//depending on the which parameter (definitions below).
//"." and ".." are not included.

#include <dirent.h>
#include <stdlib.h>
#include <stdio.h>
#include <vector>
#include <string>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include "dir.h"

//based on code from http://cermics.enpc.fr/~ts/C/FUNCTIONS/function.ref.html
vector<string> getDirList(string directory, int which) {
  vector<string> results;  
  DIR * dir_p;
  struct dirent * dir_entry_p;
  struct stat curStat;

  //Open the directory
  dir_p = opendir(directory.c_str());

  // read each entry until NULL and add to list.
  while( NULL != (dir_entry_p = readdir(dir_p)))  {
    string fullName = (normalizeDir(directory) + string(dir_entry_p->d_name));

    //printf("getDirList%d (%s): %s \n", which, directory.c_str(), fullName.c_str());
    //go to next file if we can't read this one's stats
    if (stat(fullName.c_str(), &curStat) == -1) {
      printf("error reading directory entry %s. (%s) continuing...\n", 
	     fullName.c_str(), strerror(errno));
      continue;
    }
  
    //add to list if appropriate
    if (S_ISDIR(curStat.st_mode) && which == GETDIRS) {
      if(strcmp(dir_entry_p->d_name, ".") && strcmp(dir_entry_p->d_name, "..")) {
	results.push_back(fullName);
	//printf("getDirList added %s\n", fullName.c_str());
      }
    }
    else if (S_ISREG(curStat.st_mode)  && which == GETFILES) {
      //printf("%s \n", fullName.c_str());
      results.push_back(fullName);
    }
  }

  closedir(dir_p);

  return results;
}

//makes sure there is a trailing slash on each directory
string normalizeDir(string dir) {
  if (dir[dir.length() - 1] != '/')
    dir = dir + "/";

  return dir;
}

//strips off the rightmost directory in the path and returns it.
//if called on root, does nothing.
string getParentDir(string dir) {
  int i;

  //erase rightmost directory if it is not root
  if (dir.length() > 1) {
    //make sure the end has a '/' so we can remove it
    dir = normalizeDir(dir);
    dir.erase(dir.length() - 1);
    
    //printf("end gone: %s\n", dir.c_str());
    
    i = dir.rfind("/");
    dir.erase(i + 1, dir.length());
    
    //printf("erased at %d: %s\n", i, dir.c_str());
  }

  return dir;
}

//strips the path from dir and returns simply the filename
string getBaseName(string dir) {
  int i;

  //erase rightmost directory if it is not root
  if (dir.length() > 1) {
    //make sure the end has a '/' so we can remove it
    dir = normalizeDir(dir);
    dir.erase(dir.length() - 1);
    
    //printf("end gone: %s\n", dir.c_str());
    
    i = dir.rfind("/");
    dir.erase(0, i + 1);
    
    //printf("erased at %d: %s\n", i, dir.c_str());
  }

  return dir;
}

//returns true if file is a directory
bool isDir(string file) {
  struct stat curStat;

 if (stat(file.c_str(), &curStat) == -1) {
   printf("error reading directory entry %s. (%s) continuing...\n", 
	  file.c_str(), strerror(errno));
   return false;
 }
 
 if (S_ISDIR(curStat.st_mode))
   return true;
 else
   return false;

}
